/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2012 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.

 compile:
nvcc -I/home/klaus/workspace/QuantLib -I. -O3 -arch=compute_20 -o benchmark benchmark.cu  gpubrownianbridge.o -L/home/klaus/workspace/QuantLib/ql/.libs -lQuantLib -lcurand -lboost_system -lboost_chrono
*/

#include <gpubrownianbridge.hpp>

#include <boost/chrono.hpp>
#include <stdio.h>

using namespace QuantLib;

int main(void)
{
    for (Size k=1; k < 16384ul; k*=2ul) {
        const Size steps = k;
        const Size factors = 1ul;
        const Size paths = (256ul*65536ul)/k;
        const Size length = paths*steps*factors;
        
        thrust::device_vector<real> input(length);
        thrust::device_vector<real> output(length);
        
        GPUBrownianBridge gpubb(factors, steps);
        
        const boost::chrono::high_resolution_clock::time_point t0
            = boost::chrono::high_resolution_clock::now();
        
        for (Size i=0; i < 100; ++i) {
            gpubb.transform(input.begin(), input.end(),  output.begin());
            cudaThreadSynchronize();        
        }
        
        const Real p1 = boost::chrono::duration<double>(
            boost::chrono::high_resolution_clock::now()-t0).count()/100;
        
        SobolBrownianGenerator gen(factors, steps, 
                                   SobolBrownianGenerator::Diagonal,
                                   0ul, SobolRsg::JoeKuoD6);

        const boost::chrono::high_resolution_clock::time_point t2
            = boost::chrono::high_resolution_clock::now();
        for (Size i=0; i < 10; ++i)
            for (Size j=0; j < paths; ++j) {
                gen.nextPath();
            }
        
        const Real p2 = boost::chrono::duration<double>(
            boost::chrono::high_resolution_clock::now()-t2).count()/10;
        
        printf("path length %lu speed up %f\n",k, p2/p1);
    }
}
