/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2012 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.
*/

#include <ql/models/marketmodels/browniangenerators/sobolbrowniangenerator.hpp>

#include <thrust/device_vector.h>
#include <curand.h>
#include <cudatypes.hpp>

namespace QuantLib {

    class GPUBrownianBridge {
      public:
        GPUBrownianBridge(Size factors, Size steps, 
                          SobolBrownianGenerator::Ordering ordering
                              = SobolBrownianGenerator::Diagonal,
                          unsigned long long seed = 0ull,
                          unsigned long long offset = 0ull, 
                          curandRngType rngType = CURAND_RNG_QUASI_SOBOL32);

        void transform(thrust::device_vector<real>::iterator begin,
                       thrust::device_vector<real>::iterator end,
                       thrust::device_vector<real>::iterator output);

        ~GPUBrownianBridge();

      private:
        const Size factors_, steps_;
        const boost::shared_ptr<BrownianBridge> bridge_;

        const thrust::device_vector<uint32_t> bridgeIdx_, leftIdx_, rightIdx_;
        const thrust::device_vector<real> leftWeight_, rightWeight_, stdDev_;
        thrust::device_vector<uint32_t> orderedIndices_;

        curandGenerator_t gen_;
    };
}
