/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2012 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.

 compile:
nvcc -I/home/klaus/workspace/QuantLib -I. -O3 -arch=compute_20 -o validate validate.cu gpubrownianbridge.o -L/home/klaus/workspace/QuantLib/ql/.libs -lQuantLib -lcurand
*/

#include <gpubrownianbridge.hpp>

#include <stdio.h>

using namespace QuantLib;

void validate(Size steps, Size factors, Size paths, Size runs) {
    const Size length = paths*steps*factors;

    SobolBrownianGenerator gen(factors, steps, 
                               SobolBrownianGenerator::Diagonal,
                               0ul, SobolRsg::JoeKuoD6);
    InverseCumulativeRsg<SobolRsg,InverseCumulativeNormal> 
        generator(SobolRsg(factors*steps, 0, SobolRsg::JoeKuoD6),
                  InverseCumulativeNormal());
    typedef InverseCumulativeRsg<SobolRsg,
        InverseCumulativeNormal>::sample_type sample_type;

    GPUBrownianBridge gpubb(factors, steps);

    std::vector<Real> step(factors);
    thrust::device_vector<real> input(length);
    thrust::device_vector<real> output(length);
    
    for (Size l=0; l < runs; ++l) {
        gpubb.transform(input.begin(), input.end(), output.begin());
    
        for (Size i=0; i < paths; ++i) {
            gen.nextPath();
            const sample_type& sample = generator.nextSequence();

            for (Size k=0; k < steps; ++k) {
                gen.nextStep(step);

                for (Size j=0; j < factors; ++j) {
                    const Size index = j*paths + k*factors*paths + i;
                    
                    // check Brownian bridge values
                    Real a = output[index];                
                    Real b = step[j];
                    if (std::fabs(a-b)>0.0001) {
                        printf("missmatch %lu %f %f %f\n",index, a,b,a-b);
                        exit(-1);
                    }

                    // check original Sobol numbers
                    a = input[index];
                    b = sample.value[j+k*factors];
                    if (std::fabs(a-b)>0.0001) {
                        printf("missmatch %lu %f %f %f\n",index, a,b,a-b);
                        exit(-1);
                    }
                }
            }
        }
        
    }

    printf("validation passed\n");
}

int main(void)
{
    validate(66ul, 3ul, 32ul, 5ul);
    validate(5ul, 40ul, 128ul, 2ul);
    validate(200ul, 1ul, 128ul, 2ul);
}
